-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-- Written by Kimberly L. Tripp, SQLskills.com
-- All Rights Reserved.
-- 
-- For more scripts, sample code and additional training resources,
-- go to http://www.SQLskills.com
--	Immerse yourself in SQL Server
--
-- Disclaimer - Thoroughly test this script, execute at your own risk.
-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

-- This script will help you see the impact of changing database collations.

CREATE DATABASE TestCollation
go
USE TestCollation
go
SELECT DATABASEPROPERTYEX('TestCollation', 'Collation')
	-- the default for us english without changes at the windows layer or changes during SQL Server install is:
	-- SQL_Latin1_General_CP1_CI_AS

	-- To force it use:
-- 		CREATE DATABASE TestCollation
-- 		COLLATE SQL_Latin1_General_CP1_CI_AS

go
-- To see the list of collations use the system function
SELECT * FROM ::fn_helpcollations()
go

-- Since this started from a question on how can I CHANGE collations here's how.
-- Let's change the collation to a case sensitive collation...
ALTER DATABASE TestCollation
	COLLATE Latin1_General_CS_AS_KS_WS
go

-- Confirm the change?
SELECT DATABASEPROPERTYEX('TestCollation', 'Collation')
go

-- Create some objects 
CREATE TABLE dbo.Test
(	Col1	int		NOT NULL)
go

CREATE TABLE dbo.test
(	Col1	int		NOT NULL)
go

CREATE TABLE dbo.TEST
(	Col1	int		NOT NULL)
go

-- Now try to change back to case insensitive...
ALTER DATABASE TestCollation
	COLLATE SQL_Latin1_General_CP1_CI_AS
go

-- This will generate these helpful messages:
-- Server: Msg 1505, Level 16, State 1, Line 1
-- CREATE UNIQUE INDEX terminated because a duplicate key was found for index ID 2. Most significant primary key is 'test'.
-- Server: Msg 3436, Level 16, State 1, Line 1
-- Cannot rebuild index for the 'sysobjects' table in the 'TestCollation' database.
-- Server: Msg 5072, Level 16, State 1, Line 1
-- ALTER DATABASE failed. The default collation of database 'TestCollation' cannot be set to SQL_Latin1_General_CP1_CI_AS.

-- For more info check out BOL under
--     COLLATE
--      "Specifying Collations"

-- fyi...

-- To see the server's setting
sp_helpsort

-- To see the database's setting
sp_helpdb dbname

-- To see the table's setting (for each column)
sp_help tname
