-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-- Written by Kimberly L. Tripp, SQLskills.com
-- All Rights Reserved.
-- 
-- For more scripts, sample code and additional training resources,
-- go to http://www.SQLskills.com
--	Immerse yourself in SQL Server
--
-- Disclaimer - Thoroughly test this script, execute at your own risk.
-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

IF DATABASEPROPERTYEX('JunkDB', 'COLLATION') IS NOT NULL -- Database exists so bail out!
BEGIN
	--RAISERROR ('Database already exists. If you want to remove it - comment out this line and uncomment the following line.', 16, 127)
	-- NOTE: State 127 will not kill the connection in QA... using osql for automation is safer!
	DROP DATABASE [JunkDB]
END

SET NOCOUNT ON
go

CREATE DATABASE [JunkDB]
	ON PRIMARY 
	(NAME = N'JunkDBData', 
	FILENAME = N'C:\Program Files\Microsoft SQL Server\mssql\data\JunkDBData.MDF' , 
	SIZE = 10,
	FILEGROWTH = 10%,
	MAXSIZE = 15)
LOG ON 
	(NAME = N'JunkDBLog', 
	FILENAME = N'C:\Program Files\Microsoft SQL Server\mssql\data\JunkDBLog.LDF' , 
	SIZE = 10, FILEGROWTH = 10%)
--COLLATE collation name
--SELECT * FROM ::fn_helpcollations()
go

ALTER DATABASE [JunkDB] ADD FILEGROUP [fg] 
go
ALTER DATABASE [JunkDB] 
	ADD FILE
		(NAME = N'JunkDBFile1', 
		FILENAME = N'C:\Program Files\Microsoft SQL Server\mssql\data\JunkDBFile1.NDF' , 
		SIZE = 10, FILEGROWTH = 10%) 
		TO FILEGROUP [fg]
go
ALTER DATABASE [JunkDB] 
	ADD FILE
		(NAME = N'JunkDBFile2', 
		FILENAME = N'C:\Program Files\Microsoft SQL Server\mssql\data\JunkDBFile2.NDF' , 
		SIZE = 10, FILEGROWTH = 10%) 
		TO FILEGROUP [fg]
go
ALTER DATABASE [JunkDB] 
	ADD FILE
		(NAME = N'JunkDBFile3', 
		FILENAME = N'C:\Program Files\Microsoft SQL Server\mssql\data\JunkDBFile3.NDF' , 
		SIZE = 10, FILEGROWTH = 10%) 
		TO FILEGROUP [fg]
go
USE JunkDB
go
sp_helpfile
go

CREATE TABLE dbo.DemoTable
(
	col1	int	 identity(100,10),
	col2	datetime default current_timestamp,
	col3	datetime default getdate(),
	col4	char(30) default suser_name(),
	col5	char(30) default user_name(),
	col6	char(80) default 'Now is the time for all good men to come to the aid of their country.'
) ON [FG]
--TEXTIMAGE_ON [fgname]
go
select count(*) from dbo.DemoTable
go

while 1=1
insert dbo.DemoTable default values
go
ALTER DATABASE JunkDB
	MODIFY FILEGROUP [fg] DEFAULT
go
ALTER DATABASE JunkDB
	MODIFY FILEGROUP [fg] READONLY
-- don't recommend setting this...up to and including SS2k sp3
-- read sql mag article (www.sqlmag.com) in sept 2002 InstantDoc #26067 
-- sp3???
go
ALTER DATABASE JunkDB
	MODIFY FILEGROUP [fg] READWRITE
go
ALTER DATABASE [JunkDB] ADD FILEGROUP [fgtest] 
go
ALTER DATABASE [JunkDB] 
	ADD FILE
		(NAME = N'JunkDBTestFile1', 
		FILENAME = N'C:\Program Files\Microsoft SQL Server\mssql\data\JunkDBTestFile1.NDF' , 
		SIZE = 10, FILEGROWTH = 10%) 
		TO FILEGROUP [fgtest]
go
ALTER DATABASE [JunkDB] 
	ADD FILE
		(NAME = N'JunkDBTestFile2', 
		FILENAME = N'C:\Program Files\Microsoft SQL Server\mssql\data\JunkDBTestFile2.NDF' , 
		SIZE = 10, FILEGROWTH = 10%) 
		TO FILEGROUP [fgtest]
go
sp_help Demotable -- notice that it shows on fg 
go
--DROP INDEX demotable.foo
CREATE UNIQUE CLUSTERED INDEX foo 
	ON Demotable(Col1)
	ON fgtest
go
CREATE UNIQUE INDEX bar
	ON Demotable(Col1)
	ON fg
go

sp_help Demotable -- notice that it now shows on fgtest
go
-- Let's make a filegroup for the DEFAULT objects.
-- Default objects are objects created where the filegroup 
-- name is NOT specified. By default, the default filegroup
-- is the same as the PRIMARY filegroup and you might want
-- ONLY the system tables (the database catalog) to be on the 
-- PRIMARY filegroup...
ALTER DATABASE JunkDB
	MODIFY FILEGROUP [fg] DEFAULT
go
CREATE TABLE dbo.DemoTable2
(
	col1	int	 identity(100,10),
	col2	datetime default current_timestamp,
	col3	datetime default getdate(),
	col4	char(30) default suser_name(),
	col5	char(30) default user_name(),
	col6	char(80) default 'Now is the time for all good men to come to the aid of their country.'
) -- NOTICE that I'm not specifying the filegroup that this obj
-- should be created on...
go
sp_help demotable2 -- this will show as filegroup fg because
			-- fg is the DEFAULT filegroup now.
go
-- OK - so let's review what we have...
/* ********************************************************
 Database: JunkDB
       Consists of 7 files
	JunkDBData	      PRIMARY
	JunkDBFile1	      Filegroup = fg, PROPERTY: DEFAULT
	JunkDBFile2	      Filegroup = fg, PROPERTY: DEFAULT
	JunkDBFile3	      Filegroup = fg, PROPERTY: DEFAULT
	JunkDBTestFile1     Filegroup = fgtest, PROPERTY: READWRITE
	JunkDBTestFile2     Filegroup = fgtest, PROPERTY: READWRITE
	JunkDBLog	      Transaction Log file
Next - let's try the Shrinking stuff... First let's create the 
scenario where Filegroup fg is too big...right now we have quite 
a bit of space defined BUT not all of it used. We can Shrink and 
empty the JunkDBFile3 file and then alter and remove it.
******************************************************** */
--Add some data to Demotable2 
while 1=1
insert dbo.DemoTable2 default values
go

DBCC SHRINKFILE(JunkDBFile3, EmptyFile)
go
ALTER DATABASE JunkDB
	REMOVE FILE JunkDBFile3
go
sp_helpfile -- And then there were 6 :).
go
-- Now let's try and shrink the entire database so that 
-- there is only 50% free space for each file of the database
DBCC SHRINKDATABASE (JunkDB, 75)
go
-- Then run 
sp_helpfile -- ...see the files are smaller...WAIT....NO - they're NOT!
-- DBCC SHRINKDATABASE does not allow you to shrink below
-- the initially created size. Here's a good section from the BOL.
/* **************************************
This is a straight copy from the "Remarks" section of
	DBCC SHRINKDATABASE

The target size for data and log files as calculated by 
DBCC SHRINKDATABASE can never be smaller than the 
minimum size of a file. The minimum size of a file is the 
size specified when the file was originally created, or the 
last explicit size set with a file size changing operation 
such as ALTER DATABASE with the MODIFY FILE option 
or DBCC SHRINKFILE. For example, if all the data and log 
files of mydb were specified to be 10 MB at the time 
CREATE DATABASE was executed, the minimum size of 
each file is 10 MB. DBCC SHRINKDATABASE cannot shrink 
any of the files smaller than 10 MB. If one of the files is 
explicitly grown to a size of 20 MB by using ALTER 
DATABASE with the MODIFY FILE option, the new minimum 
size of the file is 20 MB. To shrink a file to a size smaller 
than its minimum size, use DBCC SHRINKFILE and specify 
the new size. 
**************************************  */
go
-- So can we shrink to a size smaller than the initially created size!
-- Yes...but only file by file with DBCC SHRINKFILE...
-- For simplicity we'll shrink both of the fgtest files to 5 MB each
-- instead of their initially created size of 10MB (each).
DBCC SHRINKFILE(N'JunkDBTestFile1', 5)
DBCC SHRINKFILE(N'JunkDBTestFile2', 5)
go
sp_helpfile
go
-- Shrinking the Transaction log... may take a little bit more 
-- strategy! Check out these sections from the BOL 
-- in this order (so that everything makes sense):
	--Checkpoints and the Active Portion of the Log
	--Shrinking the Transaction Log