-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-- Written by Kimberly L. Tripp, SQLskills.com
-- All Rights Reserved.
-- 
-- For more scripts, sample code and additional training resources,
-- go to http://www.SQLskills.com
--	Immerse yourself in SQL Server
--
-- Disclaimer - Thoroughly test this script, execute at your own risk.
-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

-------------------------------------------------------
-- SQL Server 2005 Beta 2
-- Partitioned Tables Exercise - NOT THE FINAL VERSION
-- Please check msdn for the latest version of both the WHITEPAPER
-- AND THESE ASSOCIATED SCRIPTS!
-- May 2004

-- PartitionedTable.sql
-------------------------------------------------------

USE SalesDB
GO

-------------------------------------------------------
-- Create the partition function
-------------------------------------------------------
CREATE PARTITION FUNCTION TwoYearDateRangePFN(datetime)
AS 
RANGE LEFT FOR VALUES ('20021031 23:59:59.997',		-- Oct 2002
			'20021130 23:59:59.997',	-- Nov 2002
			'20021231 23:59:59.997',	-- Dec 2002
			'20030131 23:59:59.997',	-- Jan 2003
			'20030228 23:59:59.997',	-- Feb 2003
			'20030331 23:59:59.997',	-- Mar 2003
			'20030430 23:59:59.997',	-- Apr 2003
			'20030531 23:59:59.997',	-- May 2003
			'20030630 23:59:59.997',	-- Jun 2003
			'20030731 23:59:59.997',	-- Jul 2003
			'20030831 23:59:59.997',	-- Aug 2003
			'20030930 23:59:59.997',	-- Sep 2003
			'20031031 23:59:59.997',	-- Oct 2003
			'20031130 23:59:59.997',	-- Nov 2003
			'20031231 23:59:59.997',	-- Dec 2003
			'20040131 23:59:59.997',	-- Jan 2004
			'20040228 23:59:59.997',	-- Feb 2004
			'20040331 23:59:59.997',	-- Mar 2004
			'20040430 23:59:59.997',	-- Apr 2004
			'20040531 23:59:59.997',	-- May 2004
			'20040630 23:59:59.997',	-- Jun 2004
			'20040731 23:59:59.997',	-- Jul 2004
			'20040831 23:59:59.997',	-- Aug 2004
			'20040930 23:59:59.997')	-- Sep 2004
GO

-------------------------------------------------------
-- Create the partition scheme
-------------------------------------------------------
CREATE PARTITION SCHEME [TwoYearDateRangePScheme]
AS 
PARTITION [TwoYearDateRangePFN] TO 
		( [FG1], [FG2], [FG3], [FG4], [FG5], [FG6], 
		  [FG7], [FG8], [FG9], [FG10],[FG11],[FG12],
		  [FG13],[FG14],[FG15],[FG16],[FG17],[FG18],
		  [FG19],[FG20],[FG21],[FG22],[FG23],[FG24], 
		  [PRIMARY])
-- The last partition will ALWAYS be empty using the Rolling Range Scenario. 
-- Using the PRIMARY is acceptible for this as no data will actually reside there.
GO

-------------------------------------------------------
-- Create the Orders table on the RANGE partition scheme
-------------------------------------------------------
CREATE TABLE SalesDB.[dbo].[Orders]  
(
	[OrderID] [int] NOT NULL,
	[EmployeeID] [int] NULL,
	[VendorID] [int] NULL,
	[TaxAmt] [money] NULL,
	[Freight] [money] NULL,
	[SubTotal] [money] NULL,
	[Status] [tinyint] NOT NULL ,
	[RevisionNumber] [tinyint] NULL ,
	[ModifiedDate] [datetime] NULL ,
	[ShipMethodID]	tinyint NULL,
	[ShipDate] [datetime] NOT NULL, 
	[OrderDate] [datetime] NOT NULL
		CONSTRAINT OrdersRangeYearCK
			CHECK ([OrderDate] >= '20021001' 
				AND [OrderDate] < '20041001'), 
	[TotalDue] [money] NULL
) ON TwoYearDateRangePScheme(OrderDate)
GO

CREATE TABLE [dbo].[OrderDetails](
	[OrderID] [int] NOT NULL,
	[LineNumber] [smallint] NOT NULL,
	[ProductID] [int] NULL,
	[UnitPrice] [money] NULL,
	[OrderQty] [smallint] NULL,
	[ReceivedQty] [float] NULL,
	[RejectedQty] [float] NULL,
	[OrderDate] [datetime] NOT NULL
		CONSTRAINT OrderDetailsRangeYearCK
			CHECK ([OrderDate] >= '20021001' 
				 AND [OrderDate] < '20041001'), 
	[DueDate] [datetime] NULL,
	[ModifiedDate] [datetime] NOT NULL 
		CONSTRAINT [OrderDetailsModifiedDateDFLT] 
			DEFAULT (getdate()),
	[LineTotal]  AS (([UnitPrice]*[OrderQty])),
	[StockedQty]  AS (([ReceivedQty]-[RejectedQty]))
) ON TwoYearDateRangePScheme(OrderDate)
GO

-------------------------------------------------------
-- Add the data to the Orders and OrderDetails tables
-- THIS DATA DOES NOT EXIST
-------------------------------------------------------

-- INSERT INTO dbo.Orders 
-- SELECT * FROM dbo.CurrentOrdersTable -- this object does not exist
-- GO
-- 
-- INSERT INTO dbo.OrderDetails
-- SELECT * FROM dbo.CurrentOrderDetailsTable -- this object does not exist
-- GO

-- IF YOU JUST WANT TO SEE A BIT OF DATA... Use the two following
-- queries to populate the SalesDB tables with some data from
-- the new AdventureWorks database. Note: there are a few partitions
-- that will NOT contain any data.

INSERT dbo.[Orders]
	SELECT [PurchaseOrderID], 
			[EmployeeID], 
			[VendorID], 
			[TaxAmt], 
			[Freight], 
			[SubTotal], 
			[Status], 
			[RevisionNumber], 
			[ModifiedDate], 
			[ShipMethodID], 
			[ShipDate], 
			[OrderDate], 
			[TotalDue] 
	FROM AdventureWorks.Purchasing.PurchaseOrderHeader
		WHERE ([OrderDate] >= '20021001' 
				 AND [OrderDate] < '20041001')
-------------------------------------------------------
-- Verify Partition Ranges
-------------------------------------------------------

-- Note: Partitions 4,5,6, and 8 do NOT contain any data.

SELECT $partition.TwoYearDateRangePFN(OrderDate) 
			AS 'Partition Number'
	, min(OrderDate) AS 'Min Order Date'
	, max(OrderDate) AS 'Max Order Date'
	, count(*) AS 'Rows In Partition'
FROM Orders
GROUP BY $partition.TwoYearDateRangePFN(OrderDate)
ORDER BY 'Partition Number'
GO

-------------------------------------------------------
-- To see the partition information row by row
-------------------------------------------------------
SELECT OrderDate, 
	$partition.TwoYearDateRangePFN(OrderDate) 
		AS 'Partition Number'
FROM Orders
ORDER BY OrderDate
GO

-------------------------------------------------------
-- Create the clustered index on the partitioned table
-------------------------------------------------------
CREATE CLUSTERED INDEX OrdersCLInd
ON Orders(OrderDate, OrderID)
ON TwoYearDateRangePScheme(OrderDate)
GO
